/*****************************************************************************\
*              efs - General purpose Embedded Filesystem library              *
*          --------------------- -----------------------------------          *
*                                                                             *
* Filename : lpc2000_spi.c                                                     *
* Description : This  contains the functions needed to use efs for        *
*               accessing files on an SD-card connected to an LPC2xxx.        *
*                                                                             *
* This library is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU Lesser General Public                  *
* License as published by the Free Software Foundation; either                *
* version 2.1 of the License, or (at your option) any later version.          *
*                                                                             *
* This library is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           *
* Lesser General Public License for more details.                             *
*                                                                             *
*                                                    (c)2005 Martin Thomas    *
*                                                                             *
\*****************************************************************************/

/*
	2006, Bertrik Sikken, modified for LPCUSB
*/


/*****************************************************************************/

// *******************************
// CodeRed - port to RDB1768 board
// *******************************

// Code used in Michael Kilzer's (mjk277) Cornell University MEng Project.

#include "spi.h"



/*****************************************************************************/

/*****************************************************************************/

// Utility-functions which does not toogle CS.
// Only needed during card-init. During init
// the automatic chip-select is disabled for SSP

static U8 my_SPISend(U8 outgoing)
{
	LPC_SPI->SPDR = outgoing;
	while (!(LPC_SPI->SPSR & (1 << SPIF)));
	return LPC_SPI->SPDR;
}

/*****************************************************************************/

void SPISetSpeed(U8 speed)
{
	speed &= 0xFE;
	if (speed < SPI_PRESCALE_MIN) {
		speed = SPI_PRESCALE_MIN;
	}
	SPI_PRESCALE_REG = speed;
}


void ms_delay(int n)
{
   volatile int d;
   for (d=0; d<n*3000; d++){}
}

void SPIInit(void)
{
	U8 i;
	
// ** CodeRed - new SPI setup code	
	
	// Turn on the power
	LPC_SC->PCONP |= (1<<8); // PCSPI

	// CodeRed - clock
	LPC_SC->PCLKSEL0 &= ~(3<<16);  // PCLK_SPI
	LPC_SC->PCLKSEL0 |=  (1<<16);  // PCLK_periph = CCLK

	// setup GPIO
	LPC_GPIO0->FIODIR |= (1 << SPI_SCK_PIN) | (1 << SPI_MOSI_PIN) | (1 << TRANS_1_SSEL) | (1 << TRANS_2_SSEL) | (1 << TRANS_3_SSEL);
	LPC_GPIO0->FIODIR &= ~(1 << SPI_MISO_PIN);

	//// reset Pin-Functions
	// P0.15 set to SCK
	LPC_PINCON->PINSEL0 &= ~SPI_SCK_FUNCMASK;
	LPC_PINCON->PINSEL0 |=  SPI_SCK_FUNCBIT;	

	// P0.16, P0.17, P0.18 set to SSEL,MISI, MOSI
	LPC_PINCON->PINSEL1 &= ~(SPI_MOSI_FUNCMASK | SPI_MISO_FUNCMASK | TRANS_2_SSEL_FUNCMASK | TRANS_2_SSEL_FUNCMASK | TRANS_3_SSEL_FUNCMASK);
	LPC_PINCON->PINSEL1 |=  (SPI_MOSI_FUNCBIT | SPI_MISO_FUNCBIT);
	
	// enable SPI-Master	
	LPC_SPI->SPCR = (1 << MSTR) | (0 << CPOL);	// TODO: check CPOL
	
	DESEL_TRANS_1();
	DESEL_TRANS_2();
	DESEL_TRANS_3();
	
	SPISetSpeed(64);
 
}
/*****************************************************************************/

/*****************************************************************************/

U8 SPISend(U8 outgoing)
{
	U8 incoming;

	SELECT_CARD();
	LPC_SPI->SPDR = outgoing;
	while (!(LPC_SPI->SPSR & (1 << SPIF)));
	incoming = LPC_SPI->SPDR;
	UNSELECT_CARD();

	return incoming;
}


void SPISendN(U8 * pbBuf, int iLen, char chip)
{
	int i;
	if (chip == 1){
	    SELECT_CARD();
	}
	else {
		SELECT_CARD2();
	}
	for (i = 0; i < iLen; i++) {
		LPC_SPI->SPDR = pbBuf[i];
		while (!(LPC_SPI->SPSR & (1 << SPIF)));
	}
	if (chip == 1){
		UNSELECT_CARD();
	}
	else{
		UNSELECT_CARD2();
	}

}


void SPIRecvN(U8 * pbBuf, int iLen, char chip)
{
	int i;
	if (chip == 1){
	    SELECT_CARD();
	}
	else {
		SELECT_CARD2();
	}
	for (i = 0; i < iLen; i++) {
		LPC_SPI->SPDR = 0xFF;
		while (!(LPC_SPI->SPSR & (1 << SPIF)));
		pbBuf[i] = LPC_SPI->SPDR;
	}
	if (chip == 1){
		UNSELECT_CARD();
	}
	else{
		UNSELECT_CARD2();
	}

}

/*****************************************************************************/
