/* spi.h code courtesy of Code Red Technologies*/

// Code used in Michael Kilzer's (mjk277) Cornell University MEng Project.

#ifndef __SPI_H__
#define __SPI_H__

//#include "lpcusb_type.h"

#define U8 unsigned char
#define SPI_PRESCALE_MIN  8

#include "LPC17xx.h"

/*****************************************************************************/

// SP0SPCR  Bit-Definitions
#define CPHA    3
#define CPOL    4
#define MSTR    5
// SP0SPSR  Bit-Definitions
#define SPIF	7

// Code Red - update defines for RDB1768
#define SPI_IODIR			LPC_GPIO0->FIODIR
#define SPI_SCK_PIN			15		/* Clock		P0.15 out		(PINSEL0) */
#define SPI_SSEL_PIN		19		/* Card-Select	P0.16 GPIO out	(PINSEL1) */ // changed to P0.19
#define SPI_MISO_PIN		17		/* from Card	P0.17 in		(PINSEL1) */
#define SPI_MOSI_PIN		18		/* to Card		P0.18 out		(PINSEL1) */
#define SPI_SSEL2_PIN		20		/* Card-Select  P0.19 GPIO out  (PINSEL1) */

#define TRANS_1_SSEL       23
#define TRANS_2_SSEL       19   // changed to 19 from 24
#define TRANS_3_SSEL	   20

#define TRANS_1_SSEL_FUNCMASK (3 << 14) // PO.23 - PINSEL1 [15:14]
#define TRANS_2_SSEL_FUNCMASK (3 << 19) // PO.24 - PINSEL1 [17:16] // changed to 19
#define TRANS_3_SSEL_FUNCMASK (3 << 20) // P0.20 - PINSEL1 [9:8]
#define TRANS_1_NIRQ_FUNCMASK (3 << 18) // PO.25 - PINSEL1 [19:18]
#define TRANS_2_NIRQ_FUNCMASK (3 << 20) // PO.26 - PINSEL1 [21:20]

#define SPI_PINSEL			LPC_PINCON->PINSEL1
#define SPI_SCK_FUNCMASK	(3 << 30) // P0.15 - PINSEL0 [31:30]
#define SPI_SSEL_FUNCMASK	(3 << 6)  // P0.16 - PINSEL1 [1:0] // changed to 19
#define SPI_MISO_FUNCMASK	(3 << 2)  // P0.17 - PINSEL1 [3:2]
#define SPI_MOSI_FUNCMASK	(3 << 4)  // P0.18 - PINSEL1 [5:4]
#define SPI_SSEL2_FUNCMASK  (0 << 8)

#define SPI_SCK_FUNCBIT		(3 << 30)
#define SPI_SSEL_FUNCBIT	(0 << 0) // Note - SSEL needed as GPIO
#define SPI_MISO_FUNCBIT	(3 << 2)
#define SPI_MOSI_FUNCBIT	(3 << 4)
#define SPI_SSEL2_FUNCBIT   (0 << )

#define SPI_PRESCALE_REG	LPC_SPI->SPCCR

#define SEL_TRANS_1() LPC_GPIO0->FIOCLR = (1 << TRANS_1_SSEL)
#define DESEL_TRANS_1() LPC_GPIO0->FIOSET = (1 << TRANS_1_SSEL)
#define SEL_TRANS_2() LPC_GPIO0->FIOCLR = (1 << TRANS_2_SSEL)
#define DESEL_TRANS_2() LPC_GPIO0->FIOSET = (1 << TRANS_2_SSEL)
#define SEL_TRANS_3() LPC_GPIO0->FIOCLR = (1 << TRANS_3_SSEL)
#define DESEL_TRANS_3() LPC_GPIO0->FIOSET = (1 << TRANS_3_SSEL)

#define SELECT_CARD()   LPC_GPIO0->FIOCLR = (1 << SPI_SSEL_PIN)
#define UNSELECT_CARD() LPC_GPIO0->FIOSET = (1 << SPI_SSEL_PIN)

#define SELECT_CARD2()  LPC_GPIO0->FIOCLR = (1 << SPI_SSEL2_PIN)
#define UNSELECT_CARD2() LPC_GPIO0->FIOSET = (1 << SPI_SSEL2_PIN)


void SPIInit(void);
void SPISetSpeed(U8 speed);
void ms_delay(int n);

U8 SPISend(U8 outgoing);
void SPISendN(U8 *pbBuf, int iLen, char chip);
void SPIRecvN(U8 *pbBuf, int iLen, char chip);

#endif
